#  Model view by Melchior Franz. Modified to cockpit view by Anders Gidenstam.
#
#  Usage:
#    Add - -addon=ModelCockpitView to your FlightGear command line.
#    Once you have changed to the model or model cockpit view (using v/V) you
#    can step though the available AI/MP models with q/Q.
#    Note that the defaul view offset for the model cockpit view usually isn't
#    quite right, but you can move the view using MMB drag in mouse view mode.

var model_view_handler = {
    init : func {
        me.models = {};
        me.list = [];
        me.current = 0;
        me.active = 0;
    },
    start : func {
        me.models = {};
        var ai = props.globals.getNode("/ai/models", 1);
        foreach (var m; [props.globals]
                        ~ ai.getChildren("aircraft")
                        ~ ai.getChildren("carrier")
                        ~ ai.getChildren("multiplayer")
                        ~ ai.getChildren("tanker")
                        ~ ai.getChildren("groundvehicle"))
            me.models[m.getPath()] = m;

        me.lnr = [];
        append(me.lnr, setlistener("/ai/models/model-added", func(n) {
            var m = props.globals.getNode(n.getValue(), 1);
            me.models[m.getPath()] = m;
        }));
        append(me.lnr, setlistener("/ai/models/model-removed", func(n) {
            var m = props.globals.getNode(n.getValue(), 1);
            delete(me.models, m.getPath());
        }));
        me.active = 1;
        me.reset();
    },
    update : func {
        return 0;
    },
    stop : func {
        me.active = 0;
        foreach (var listener; me.lnr)
            removelistener(listener);
    },
    reset : func {
        me.next(me.current);
    },
    next : func(v) {
        if (!me.active or !size(me.models))
            return;
        if (v)
            me.current += v;
        else
            me.current = 0;
      
        me.list = sort(keys(me.models), cmp);
        if (me.current < 0)
            me.current = size(me.list) - 1;
        elsif (me.current >= size(me.list))
            me.current = 0;

        var s = "/sim/view[97000]/config";
        var c = me.list[me.current];
        setprop(s, "eye-lat-deg-path", c ~ "/position/latitude-deg");
        setprop(s, "eye-lon-deg-path", c ~ "/position/longitude-deg");
        setprop(s, "eye-alt-ft-path", c ~ "/position/altitude-ft");
        setprop(s, "eye-heading-deg-path", c ~ "/orientation/true-heading-deg");
        setprop(s, "eye-pitch-deg-path", c ~ "/orientation/pitch-deg");
        setprop(s, "eye-roll-deg-path", c ~ "/orientation/roll-deg");

        var n = me.models[me.list[me.current]];
        var type = n.getName();
        var name = nil;
        if (type == "") {
            if (name = getprop("/sim/multiplay/callsign"))
                 name = 'callsign "' ~ name ~ '"';
        } else {
            if ((name = n.getNode("callsign")) != nil and (name = name.getValue()))
                name = 'callsign "' ~ name ~ '"';
            elsif ((name = n.getNode("name")) != nil and (name = name.getValue()))
                name = n.getName() ~ ' "' ~ name ~ '"';
            else
                name = n.getName() ~ " " ~ n.getIndex();
        }
        var color = {};
        if (type != "multiplayer")
            color = { text: { color: { red: 0.5, green: 0.8, blue: 0.5 }}};
        if (name)
            gui.popupTip(name, 2, color);
    },
};

var main = func {
    globals["model-cockpit-view"] = {};
    globals["model-cockpit-view"].model_view_handler = model_view_handler;

    settimer(func {
        view.manager.register("Model Cockpit View",
                              model_view_handler);
    }, 1);
}
